﻿//------------------------------------------------------------------------------
//	FFT クラス（データが実数の場合），CortexM4 C++ 用クラス FftReal に準拠
//		作成者： 三上直樹， 2014/04/02 作成，(c)三上直樹
//------------------------------------------------------------------------------

using System.Windows.Forms;
using System.Numerics;

namespace Mikami.FFT
{
	public class FftReal
	{
		private readonly int	N_FFT_;
		private Complex[]		wTable_;	// 回転子用
		private int[]			bTable_;	// ビット逆順用
		private Complex[]		u_;			// 作業領域

		// コンストラクタ
		public FftReal(int n)
		{
			N_FFT_ = n;
			int pw2 = n;
			while ((pw2 & 0x1) == 0) pw2 = pw2 >> 1;
			if (pw2 != 0x1) MessageBox.Show("データ数が２のべき乗ではありません");

			wTable_ = new Complex[n/2];
			bTable_ = new int[n];
			u_ = new Complex[n];

			// 回転子の計算
			Complex arg = new Complex(0, -6.28318531f/n);
			for (int k=0; k<n/2; k++) wTable_[k] = Complex.Exp(arg*k);

			// ビット逆順用
			int nHalf = n/2;
			bTable_[0] = 0;
			for (int m=1; m<n; m*=2)
			{
				for (int k=0; k<m; k++) bTable_[k+m] = bTable_[k] + nHalf;
				nHalf = nHalf/2;
			}
		}

		// FFT の実行
		public void Execute(double[] x, Complex[] y)
		{
			for (int n=0; n<N_FFT_; n++) u_[n] = new Complex(x[n], 0);

			// 最後の段を除く処理
			int nHalf = N_FFT_/2;
			for (int stg=1; stg<N_FFT_/2; stg*=2)
			{
				int nHalf2 = nHalf*2;
				for (int kp=0; kp<N_FFT_; kp+=nHalf2)
				{
					int kx = 0;
					for (int k=kp; k<kp+nHalf; k++)
					{
						// バタフライ演算
						Complex utmp = u_[k+nHalf];
						u_[k+nHalf] = (u_[k] - utmp)*wTable_[kx];
						u_[k] = u_[k] + utmp;
						kx = kx + stg;
					}
				}
				nHalf = nHalf/2;
			}

			// 最後の段の処理
			y[0] = u_[0] + u_[1];
			y[N_FFT_/2] = u_[0] - u_[1];
			for (int k=2; k<N_FFT_; k+=2) u_[k] = u_[k] + u_[k+1];
			// ビット逆順の並べ替え
			for (int k=1; k<N_FFT_/2; k++) y[k] = u_[bTable_[k]];
		}
	}
}
