﻿//------------------------------------------------------------------------------
//	窓掛け用のクラス
//		ハニング窓，ハミング窓，ブラックマン窓
//		作成者： 三上直樹， 2014/05/05 作成，(c)三上直樹
//------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Mikami.FFT
{
	class Window
	{
		private readonly int N_DATA_;
		private double[] w_;

		public enum Type { Hanning, Hamming, Blackman };

		// コンストラクタ
		public Window(int nData, Type myType)
		{
			N_DATA_ = nData;
			w_ = new double[nData];
			double pi2N = 2.0*Math.PI/(double)nData;
			double pi4N = 4.0*Math.PI/(double)nData;

			Func<int, double> Hanning = n => 0.5 - 0.5*Math.Cos(pi2N*n); 
			Func<int, double> Hamming = n => 0.54 - 0.46*Math.Cos(pi2N*n); 
			Func<int, double> Blackman = n => 0.42 - 0.5*Math.Cos(pi2N*n) + 0.08*Math.Cos(pi4N*n);

			switch (myType)
			{
				case Type.Hanning  : Generate(Hanning); break;
				case Type.Hamming  : Generate(Hamming); break;
				case Type.Blackman : Generate(Blackman); break;
			}
		}

		private void Generate(Func<int, double> Window)
		{
			for (int n=0; n<N_DATA_; n++) w_[n] = Window(n);
		}

		// 窓掛けの実行
		public void Execute(double[] xn, double[] xnWn)
		{
			for (int n=0; n<N_DATA_; n++) xnWn[n] = w_[n]*xn[n];
		}
	}
}
